# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"remove_dsstore": "00_core.ipynb",
         "get_basename": "04_dataloader.ipynb",
         "IMAGENET_LABELS": "00_core.ipynb",
         "tf": "01_image.ipynb",
         "torch": "01_image.ipynb",
         "DATA_FORMATS": "01_image.ipynb",
         "DEFAULT_MODE": "01_image.ipynb",
         "read_image": "01_image.ipynb",
         "resize_image": "01_image.ipynb",
         "BoundingBoxes": "01_image.ipynb",
         "Chitra": "01_image.ipynb",
         "benchmark": "02_datagenerator.ipynb",
         "get_filenames": "02_datagenerator.ipynb",
         "get_label": "02_datagenerator.ipynb",
         "ImageSizeList": "02_datagenerator.ipynb",
         "Pipeline": "02_datagenerator.ipynb",
         "Dataset": "02_datagenerator.ipynb",
         "MODEL_DICT": "03_trainer.ipynb",
         "OPT_DICT": "03_trainer.ipynb",
         "create_classifier": "03_trainer.ipynb",
         "create_cnn": "03_trainer.ipynb",
         "Trainer": "03_trainer.ipynb",
         "InterpretModel": "03_trainer.ipynb",
         "Learner": "03_trainer.ipynb",
         "AUTOTUNE": "04_dataloader.ipynb",
         "show_batch": "04_dataloader.ipynb",
         "Clf": "04_dataloader.ipynb",
         "label_color": "05_visualization.ipynb",
         "colors": "05_visualization.ipynb",
         "draw_box": "05_visualization.ipynb",
         "draw_caption": "05_visualization.ipynb",
         "draw_boxes": "05_visualization.ipynb",
         "draw_detections": "05_visualization.ipynb",
         "draw_annotations": "05_visualization.ipynb",
         "disable_gpu": "06_tf_utils.ipynb",
         "limit_gpu": "06_tf_utils.ipynb",
         "gpu_dynamic_mem_growth": "06_tf_utils.ipynb",
         "is_installed": "07_import_utils.ipynb",
         "INSTALLED_MODULES": "07_import_utils.ipynb",
         "pytorch_to_onnx": "08_model_converter.ipynb",
         "onnx_to_pytorch": "08_model_converter.ipynb",
         "tf2_to_onnx": "08_model_converter.ipynb",
         "tf2_to_pytorch": "08_model_converter.ipynb"}

modules = ["core.py",
           "image.py",
           "datagenerator.py",
           "trainer.py",
           "dataloader.py",
           "visualization.py",
           "utility/tf_utils.py",
           "utility/import_utils.py",
           "converter/core.py"]

doc_url = "https://chitra.aniketmaurya.com/"

git_url = "https://github.com/aniketmaurya/chitra/tree/master/"

def custom_doc_links(name): return None
